﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Windows.Threading;
using IncrementingLineSeries.Annotations;

namespace IncrementingLineSeries
{
    public class MainViewModel : INotifyPropertyChanged
    {
        #region unrelated 
        private readonly DispatcherTimer timer;
        private int incrementingInt;
        #endregion
        
        public MainViewModel()
        {
            timer=new DispatcherTimer();
            timer.Interval = TimeSpan.FromMilliseconds(500);
            timer.Tick += Timer_Tick;
        }

        private ObservableCollection<ChartDataPoint> chartData;
        public ObservableCollection<ChartDataPoint> ChartData
        {
            get { return chartData ?? (chartData = new ObservableCollection<ChartDataPoint>()); }
            set { chartData = value; OnPropertyChanged();}
        }
        
        /// <summary>
        /// this method gets run every 500 milliseconds
        /// </summary>
        private void AddDataPoint()
        {
            #region unrelated

            var valueToUse = incrementingInt % 2 == 0 ? incrementingInt++ : -incrementingInt++;

            #endregion

            //create a new data point
            var nextDataPoint = new ChartDataPoint
            {
                DataTimestamp = DateTime.Now,
                DataValue = valueToUse
            };
            
            //Add the new data point to the collection, which will be rendered on the ChartView
            ChartData.Add(nextDataPoint);

            //To keep the demo from getting too large, I keep the datapoints to within 30
            //this keep the the Chart scrolling along
            if(ChartData.Count>30)
                ChartData.RemoveAt(0);
        }

        #region unrelated code

        private void Timer_Tick(object sender, EventArgs e)
        {
            Dispatcher.CurrentDispatcher.Invoke(AddDataPoint);
        }

        public void StartTimer()
        {
            timer.Start();
        }

        public void StopTimer()
        {
            timer.Stop();
        }

        #endregion

        #region INPC
        public event PropertyChangedEventHandler PropertyChanged;

        [NotifyPropertyChangedInvocator]
        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion
    }
}
